pub struct SelectorMlecheckProver<'b, P: PackedField, B: Bitwise, Data: BufferData<P> = Vec<P>> { /* private fields */ }Expand description
A SumcheckProver implementation that proves an mlecheck over many compositions of the
form selected * selector + (1 - selector), where selected is the shared large field
multilinear and selector comes from the set of 1-bit multilinears. Unlike other multi mlecheck
provers however the evaluation point is not shared but is specified per selector.
The set of 1-bit multilinears is represented by a power-of-two long slice of bitmasks, and the multilinear set is constructed by arranging the bitmasks as a 2D matrix in row-major order and taking vertical slices. This representation is very compact and has no embedding overhead.
To combat memory blowup issues arising from folding 1-bit multilinears, this prover introduces
switchover. See BinarySwitchover for more in-depth explanation of the mechanism. Also note
that the need to expand the equality indicator for each multilinear still results in some
blowup.
Implementations
Sourceimpl<'b, F: Field, P: PackedField<Scalar = F>, B: Bitwise, Data: BufferData<P>> SelectorMlecheckProver<'b, P, B, Data>
impl<'b, F: Field, P: PackedField<Scalar = F>, B: Bitwise, Data: BufferData<P>> SelectorMlecheckProver<'b, P, B, Data>
Sourcepub fn new(
selected: FieldBuffer<P, Data>,
claims: Vec<Claim<F>>,
bitmasks: &'b [B],
weights: Vec<F>,
switchover: usize,
) -> Self
pub fn new( selected: FieldBuffer<P, Data>, claims: Vec<Claim<F>>, bitmasks: &'b [B], weights: Vec<F>, switchover: usize, ) -> Self
Constructs a prover, given bitmasks as representation of 1-bit columns, selected being
the shared large field multilinear, individual claims per selector, weights to combine
the per-selector round polynomials into one (one weight per claim), and switchover as the
round at which 1-bit columns should be folded.
The prover exposes a single claim — the weights-combination Σ_i weights[i] · C_i of the
per-selector claims. Supplying the equality-indicator tensor eq_k(γ, ·) as the weights
batches the claims with eq_k(γ, i).